#include "uifragment.h"

typedef struct appdata {
	Evas_Object *win;
	Evas_Object *conform;
	Evas_Object *layout;
	Evas_Object *nf;
	Evas_Object *mainview;
} appdata_s;

static void
win_delete_request_cb(void *data , Evas_Object *obj , void *event_info)
{
	ui_app_exit();
}

static void
win_back_cb(void *data, Evas_Object *obj, void *event_info)
{
	appdata_s *ad = data;
	/* Let window go to hide state. */
	elm_win_lower(ad->win);
}

static Evas_Object* create_label(Evas_Object *parent, const char *text)
{
	Evas_Object *text_obj = elm_label_add(parent);
	//elm_object_style_set(button, "transparent");
	evas_object_size_hint_weight_set(text_obj, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(text_obj, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_object_text_set(text_obj, text);
	return text_obj;
}


static Evas_Object* create_button(Evas_Object *parent, Evas_Object *content)
{
	Evas_Object *button = elm_button_add(parent);
	evas_object_size_hint_weight_set(button, EVAS_HINT_EXPAND, 0.0);
	evas_object_size_hint_align_set(button, EVAS_HINT_FILL, 0.0);

	elm_object_content_set(button, content);

	return button;
}

static Evas_Object* create_fragment_title(Evas_Object *parent, const char *title)
{
	Evas_Object *label = create_label(parent, title);
	Evas_Object *fragment_title = create_button(parent, label);
	return fragment_title;
}

/* This function displays fragnent 1 components */

static Evas_Object* fragment1(Evas_Object *parent)
{

	/*Table*/

	Evas_Object *table = elm_table_add(parent);
	elm_table_homogeneous_set(table, EINA_TRUE); //Sets the homogeneous layout in the table.
	evas_object_size_hint_align_set(table, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_show(table);

	/* Label Widget */

		Evas_Object *label = elm_label_add(parent);
		elm_object_text_set(label, "<font_size=50><color=#006400>Label</color></font_size>");
		evas_object_show(label);
		elm_table_pack(table, label, 0, 0, 1, 1);// last 4 parameters are col,row,colspan,rowspan

		/* Favorite Widget */

		Evas_Object *icon,*check;
		check = elm_check_add(parent);
		elm_object_style_set(check, "favorite");
		icon = elm_icon_add(parent);
		elm_icon_standard_set(icon, "home");
		elm_object_part_content_set(check, "icon", icon);
		evas_object_show(check);
		elm_table_pack(table,check,1,0,1,1);

		/* Slider Widget */

		Evas_Object *slider;
		slider = elm_slider_add(parent);
		elm_object_style_set(slider, "default");
		evas_object_color_set(slider, 155, 15, 0, 255);
		evas_object_show(slider);
		elm_table_pack(table,slider,0,1,1,1);

		/* Spinner Widget */

		Evas_Object *spin;
		spin = elm_spinner_add(parent);
		elm_object_style_set(spin, "vertical");
		elm_spinner_step_set(spin, 2.0);
		elm_spinner_wrap_set(spin, EINA_TRUE);
		elm_spinner_min_max_set(spin, -25.0, 100.0);
		evas_object_show(spin);
		elm_table_pack(table,spin,1,1,1,1);

	return table;
}


/* This function displays fragment 2 components */

static Evas_Object* fragment2(Evas_Object *parent)
{
	/* Box Container */

	Evas_Object *box = elm_box_add(parent);
	elm_box_horizontal_set(box, EINA_TRUE); // For horizontal placement in box
	evas_object_size_hint_align_set(box, EVAS_HINT_FILL, 0.0);
	evas_object_show(box);

	/* Label Widget */

	Evas_Object *label = elm_label_add(parent);
	elm_object_text_set(label, "<font_size=50><color=#006400>Label1</color></font_size>");
	evas_object_show(label);
	elm_box_pack_end(box, label);

	/* Colorsel Widget */

	Evas_Object *colorsel;
	colorsel = elm_colorselector_add(parent);
	elm_colorselector_mode_set(colorsel, ELM_COLORSELECTOR_PALETTE);
	elm_colorselector_palette_name_set(colorsel, "mypalette");
	elm_colorselector_palette_color_add(colorsel, 255, 0, 0, 255);
	elm_colorselector_palette_color_add(colorsel, 0, 255, 0, 255);
	elm_colorselector_palette_color_add(colorsel, 0, 0, 255, 255);
	evas_object_show(colorsel);
	elm_box_pack_end(box, colorsel);


	return box;
}

/* This function displays fragment 3 components */

static Evas_Object* fragment3(Evas_Object *parent)
{

	/* Box Container */

	Evas_Object *box = elm_box_add(parent);
	evas_object_size_hint_align_set(box, EVAS_HINT_FILL, 0.0);
	evas_object_show(box);

	/* Radio Button Widget */

	Evas_Object *radio,*radio_group;
	radio = elm_radio_add(parent);
	elm_object_text_set(radio,"Mobile");
	elm_radio_state_value_set(radio,1);
	radio_group = radio;
	evas_object_show(radio);
	elm_box_pack_end(box, radio);

	radio = elm_radio_add(parent);
	elm_object_text_set(radio,"Wearable");
	elm_radio_state_value_set(radio,2);
	elm_radio_group_add(radio,radio_group);

	evas_object_show(radio);
	elm_box_pack_end(box, radio);

	/*DateTime Widget*/

	Evas_Object *datetime;
	datetime = elm_datetime_add(parent);
	elm_datetime_field_visible_set(datetime, ELM_DATETIME_YEAR, EINA_FALSE);
	elm_datetime_field_visible_set(datetime, ELM_DATETIME_MONTH, EINA_FALSE);
	elm_datetime_field_visible_set(datetime, ELM_DATETIME_DATE, EINA_FALSE);
	elm_object_style_set(datetime, "date_layout");
	elm_datetime_format_set(datetime, "%d%B%Y");
	evas_object_show(datetime);
	elm_box_pack_end(box, datetime);


	return box;
}


/* This function creates all the fragment  */

static Evas_Object* fragment_UI(appdata_s *ad)
{
	Evas_Object *scroller, *box_full, *table, *fragment_title, *item;

	/* Scroller is used  to see all the contents in the UI view if contents goes outside of UI visible part */
	scroller = elm_scroller_add(ad->win);
	evas_object_size_hint_weight_set(scroller, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(scroller, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_show(scroller);

	/* Parent Box */

	box_full = elm_box_add(ad->win);
	elm_box_align_set(box_full, 0,0 );
	evas_object_size_hint_weight_set(box_full, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_content_set(scroller, box_full);

	/* Fragment-1 Object */

	fragment_title = create_fragment_title(box_full, "<font_size=50><color=#FF4500FF>Fragment 1</color></font_size>");
	evas_object_show(fragment_title);
	elm_box_pack_end(box_full, fragment_title);
	table = fragment1(box_full);
	evas_object_show(table);
	elm_box_pack_end(box_full, table);	//Fragment-1 is added at the end of box

	/* Fragment-2 Object */

	fragment_title = create_fragment_title(box_full, "<font_size=50><color=#FF4500FF>Fragment 2</color></font_size>");
	evas_object_show(fragment_title);
	elm_box_pack_end(box_full, fragment_title);
	item = fragment2(box_full);
	evas_object_show(item);
	elm_box_pack_end(box_full, item);	//Fragment-2 is added at the end of box

	/* Fragment-3 Object */

	fragment_title = create_fragment_title(box_full, "<font_size=50><color=#FF4500FF>Fragment 3</color></font_size>");
	evas_object_show(fragment_title);
	elm_box_pack_end(box_full, fragment_title);
	item = fragment3(box_full);
	evas_object_show(item);
	elm_box_pack_end(box_full, item);	//Fragment-3 is added at the end of box

	return scroller;

}

static void create_base_gui(appdata_s *ad)
{
	Evas_Object *bg = NULL;

	/* Window */
	ad->win = elm_win_util_standard_add(PACKAGE, PACKAGE);
	elm_app_base_scale_set(1.8);

	elm_win_conformant_set(ad->win, EINA_TRUE);
	elm_win_autodel_set(ad->win, EINA_TRUE);

	if (elm_win_wm_rotation_supported_get(ad->win)) {
		int rots[4] = { 0, 90, 180, 270 };
		elm_win_wm_rotation_available_rotations_set(ad->win, (const int *)(&rots), 4);
	}

	evas_object_smart_callback_add(ad->win, "delete,request", win_delete_request_cb, NULL);

	/* Conformant */
	ad->conform = elm_conformant_add(ad->win);
	elm_win_indicator_mode_set(ad->win, ELM_WIN_INDICATOR_SHOW);
	elm_win_indicator_opacity_set(ad->win, ELM_WIN_INDICATOR_OPAQUE);
	evas_object_size_hint_weight_set(ad->conform, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_win_resize_object_add(ad->win, ad->conform);
	evas_object_show(ad->conform);

	/* Layout */
	ad->layout = elm_layout_add(ad->conform);
	evas_object_size_hint_weight_set(ad->layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_layout_theme_set(ad->layout, "layout", "application", "default");

	Evas_Object *bimage;
	bimage = elm_bg_add(ad->layout);
	elm_bg_color_set(bimage, 0xFF, 0x00, 0x00);
	evas_object_show(ad->layout);
	elm_object_content_set(ad->conform, ad->layout);

	/* Naviframe */
	ad->nf = elm_naviframe_add(ad->layout);
	ad->mainview = fragment_UI(ad);
	elm_naviframe_item_push(ad->nf, "UI Fragment", NULL, NULL, ad->mainview, NULL);
	elm_object_part_content_set(ad->layout, "elm.swallow.content", ad->nf);

	/* Show window after base gui is set up */
	evas_object_show(ad->win);

	eext_object_event_callback_add(ad->mainview, EEXT_CALLBACK_BACK, win_back_cb, ad);
}

static bool
app_create(void *data)
{
	/* Hook to take necessary actions before main event loop starts
		Initialize UI resources and application's data
		If this function returns true, the main loop of application starts
		If this function returns false, the application is terminated */
	appdata_s *ad = data;

	create_base_gui(ad);

	return true;
}

static void
app_control(app_control_h app_control, void *data)

{
	// Handle the launch request
}

static void
app_pause(void *data)
{
	// Take necessary actions when application becomes invisible
}

static void
app_resume(void *data)
{
	// Take necessary actions when application becomes visible.
}

static void
app_terminate(void *data)
{
	// Release all resources
}

static void
ui_app_lang_changed(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_LANGUAGE_CHANGED*/
	char *locale = NULL;
	system_settings_get_value_string(SYSTEM_SETTINGS_KEY_LOCALE_LANGUAGE, &locale);
	elm_language_set(locale);
	free(locale);
	return;
}

static void
ui_app_orient_changed(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_DEVICE_ORIENTATION_CHANGED*/
	return;
}

static void
ui_app_region_changed(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_REGION_FORMAT_CHANGED*/
}

static void
ui_app_low_battery(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_LOW_BATTERY*/
}

static void
ui_app_low_memory(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_LOW_MEMORY*/
}

int
main(int argc, char *argv[])
{
	appdata_s ad = {0,};
	int ret = 0;

	ui_app_lifecycle_callback_s event_callback = {0,};
	app_event_handler_h handlers[5] = {NULL, };

	event_callback.create = app_create;
	event_callback.terminate = app_terminate;
	event_callback.pause = app_pause;
	event_callback.resume = app_resume;
	event_callback.app_control = app_control;

	ui_app_add_event_handler(&handlers[APP_EVENT_LOW_BATTERY], APP_EVENT_LOW_BATTERY, ui_app_low_battery, &ad);
	ui_app_add_event_handler(&handlers[APP_EVENT_LOW_MEMORY], APP_EVENT_LOW_MEMORY, ui_app_low_memory, &ad);
	ui_app_add_event_handler(&handlers[APP_EVENT_DEVICE_ORIENTATION_CHANGED], APP_EVENT_DEVICE_ORIENTATION_CHANGED, ui_app_orient_changed, &ad);
	ui_app_add_event_handler(&handlers[APP_EVENT_LANGUAGE_CHANGED], APP_EVENT_LANGUAGE_CHANGED, ui_app_lang_changed, &ad);
	ui_app_add_event_handler(&handlers[APP_EVENT_REGION_FORMAT_CHANGED], APP_EVENT_REGION_FORMAT_CHANGED, ui_app_region_changed, &ad);
	ui_app_remove_event_handler(handlers[APP_EVENT_LOW_MEMORY]);

	ret = ui_app_main(argc, argv, &event_callback, &ad);
	if (ret != APP_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "ui_app_main() is failed. err = %d", ret);
	}

	return ret;
}

